/* Xmsgd
 * Copyright (C) 2001 Thomas Vander Stichele <thomas@apestaart.org>
 * 
 * fifo.c: provides server functions using FIFO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * version 0.6.0
 *
 * last changed : 27/06/2001
 */

#include "stdio.h"
#include "stdlib.h"

#include "cfg.h"
#include "textinput.h"

FILE *input;
 
extern int debug;

int start_listener (cfg_t cfg)
{
  DEBUG (("Opening %s for reading\n", cfg.fifo));
  if ((input = fopen (cfg.fifo, "r")) == NULL)
  {
    printf ("fifo.c: start_listener: error opening %s !\n", cfg.fifo);
    exit (1);
  }
  return 1;
}

int 
message_waiting (cfg_t cfg, msg_t **msgp)
{
  /* check to see if there's a message waiting 
   * returns -1 on error
   * returns 0 when none are waiting
   * returns number of waiting messages 
   */

  char line[256];
  char *line1, *line2;

  if (*msgp != NULL)
    return -1;
 
  if (fgets (line, 255, input) == NULL)
  {
    DEBUG (("fifo.c: fgets 1 returned NULL\n"));
    return 0;  
  }
  
  /* initialize msg */
  *msgp = (msg_t *) malloc (sizeof (msg_t));
  if (*msgp == NULL)
  {
    return -1;
  }

  ClearMsg (*msgp);
  /* fill in msg with defaults */
  DefaultMsg (*msgp);

  line1 = NULL;
  line2 = NULL;
  if (line[0] == '-')
  {
    /* first line is option line */
    copy_string (&line1, line, "option line");
    do {} while (fgets (line, 255, input) == NULL);
  }
  copy_string (&line2, line, "text line");
  ti_process_message (line1, line2, *msgp);

  DEBUG (("message_waiting: text %s\n",
	  (*msgp)->text));
  return 1;
}
