/* Xmsgd 
 * Copyright (C) 2001 Thomas Vander Stichele <thomas@apestaart.org>
 * 
 * textinput.c: functions to process an input message based on text-based lines
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * version 0.6.0
 *
 * last changed : 27/06/2001
 *
 * input messages are either one or two lines
 * if they're one line, it's just text
 * if they're two lines, then the first must start with '-' and contains
 * options and option values, the second line is the text
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "textinput.h"

extern int debug;

int
parse_option_line (char *line, msg_t *msgp)
{
  /* parse the line, but in reverse */
  char *lptr = NULL;

  lptr = line + strlen (line) - 1;	/* start at end of string */
  if (*lptr == '\n') { *lptr = '\0'; --lptr; }
  DEBUG (("end %d\n", *lptr));
  DEBUG (("fifo.c: pol started\n"));
  while (lptr > line)
  {
    DEBUG (("going to option\n"));
    while (lptr > line && !(*lptr == '-' && *(lptr - 1) == ' ')) --lptr;
    if (*lptr != '-')
      return -1;
    ++lptr;			/* move to option letter */

    switch (*lptr)
    {
      case 'c':
        copy_string (&(*msgp).color_name, lptr + 2, "color name");
        DEBUG (("color option : -%s-\n", msgp->color_name));
        break;
      case 'f':
        copy_string (&(*msgp).font_name, lptr + 2, "font name");
        DEBUG (("font option : -%s-\n", msgp->font_name));
        break;
      case 'i':
        copy_string (&(*msgp).icon_file, lptr + 2, "icon file");
        DEBUG (("icon option : -%s-\n", msgp->icon_file));
        break;
      case 'd':
	DEBUG (("duration\n"));
	msgp->duration = atof (lptr + 1);
	break;
      case 'l':
	DEBUG (("line\n"));
	msgp->line_number = atoi (lptr + 1);
	break;
      default:
        break;  
    }
    lptr -= 2;		/* move back to space separation */
    if (lptr > line) *lptr = '\0'; /* change space to null-terminator */
  }
  return 0;
}


int
ti_process_message (char *line1, char *line2, msg_t *msgp)
{
  if (line1 != NULL)
  {
    /* options given, process them */
    parse_option_line (line1, msgp);
  }
  free (line1);
  msgp->text = line2;
  return 0;
}

