/* Xmsgd 
 * Copyright (C) 2001 Thomas Vander Stichele <thomas@apestaart.org>
 *
 * Xmsglib.c: helper library for Xmsg and Xmsgd
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * version 0.6.0
 *
 * last changed : 27/06/2001
 *
 * functions used in both stand-alone Xmsg and Xmsgd server
 */

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* project includes */

#include "../lib/Xmsglib.h"
#include "../lib/default.h"

/* debug ? */
extern char debug;

/* experimental code */

#define NEW_CODE 1

/* create two windows, one for the icon and one for the message
 * put them next to each other
 * display the icon in the first
 * display the message graphically in the second
 */

/* function prototypes */

Window 
CreateWindow (Display* display, int screen_num, 
              int x, int y, int width, int height)
{
    /* Create a window and map it */
    
    int vmask;
    XSetWindowAttributes attribs;
    Window w;

    /* message window */
    vmask = CWSaveUnder|CWOverrideRedirect|CWColormap|CWBorderPixel;
    attribs.save_under = True;
    attribs.override_redirect = True;
    attribs.colormap = DefaultColormap (display, screen_num);
    attribs.background_pixel = 1000; 
    attribs.border_pixel = 0; /* do not care */

    w = XCreateWindow (display, DefaultRootWindow (display), x, y, 
			width, height, 0, CopyFromParent, 
			CopyFromParent, CopyFromParent, vmask, &attribs);

    XMapWindow (display, w);
    return w;
}

int 
LoadFont (Display* display, msg_t* msg)
{
  /* try to load the given font. */

  msg->font_info = XLoadQueryFont(display, msg->font_name);
  if (!msg->font_info) 
  {
      fprintf(stderr, "XLoadQueryFont: failed loading font '%s'\n", msg->font_name);
      exit(-1);
  }
  /* find the height of the characters drawn using this font.        */
  msg->font_height = msg->font_info->ascent + msg->font_info->descent;
  return 0;
}

GC
CreateGC (Display* display, Window win, char color[])
{
  /* Create a graphics context with the given colo as foreground color */
  
  GC gc;                                /* handle of newly created GC.  */
  unsigned long valuemask = 0;          /* which values in 'values'  to */
                                        /* check when creating the GC.  */
                                        /* check when creating the GC.  */
  XGCValues values;                     /* initial values for the GC.   */
  unsigned int line_width = 2;          /* line width for the GC.       */
  int line_style = LineSolid;           /* style for lines drawing and  */
  int cap_style = CapButt;              /* style of the line's edje and */
  int join_style = JoinBevel;           /*  joined lines.               */
  int screen_num = DefaultScreen(display);

  Colormap colormap;
  XColor col;
  
  gc = XCreateGC(display, win, valuemask, &values);
  if (gc < 0) 
  {
        fprintf(stderr, "XCreateGC: \n");
  }
  XSetSubwindowMode (display, gc, IncludeInferiors);
  
  /* allocate foreground and background colors for this GC. */
  colormap = DefaultColormap (display, 0);
  XParseColor (display, colormap, color, &col);
  XAllocColor (display, colormap, &col);

  XSetForeground(display, gc, col.pixel);
  XSetBackground(display, gc, BlackPixel(display, screen_num));
  
  /* define the style of lines that will be drawn using this GC. */
  XSetLineAttributes (display, gc,
                      line_width, line_style, cap_style, join_style);
  
  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle (display, gc, FillSolid);

  return gc;
}

int 
DrawText (Display* display, Window window, msg_t msg, char *text)
{
  /* display the text on the window using a shadowy effect 
   * create a shaped window for it
   */

  Pixmap shapemask;
  GC gc_shape, gc_text, gc_border;
  XGCValues gcv;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  shapemask = XCreatePixmap (display, window, msg.message_width, MSG_H, 1);
  gc_shape  = XCreateGC (display, shapemask, 0 , &gcv);
  gc_text   =  CreateGC (display, window, msg.color_name);
  gc_border =  CreateGC (display, window, "black");

  /* Do font stuff */

  /* assign the given font to our GC */
  XSetFont (display, gc_text, msg.font_info->fid);
  XSetFont (display, gc_border, msg.font_info->fid);
  XSetFont (display, gc_shape, msg.font_info->fid);

  /* clear the mask */
  XSetForeground (display, gc_shape, 0);
  XFillRectangle (display, shapemask, gc_shape, 0, 0, msg.message_width, MSG_H);
  XSetForeground (display, gc_shape, 1);
  
  {
    /* variables used for drawing the text strings */
    int x, y;
    int x_off, y_off;

    x = 0;
    y = msg.font_height;    

    /* draw black border part */
    for (x_off = 0; x_off <= 4; ++x_off)
    {
      for (y_off = 0; y_off <= 4; ++y_off)
      {
        /* draw both border text and shape mask text */
        XDrawString (display, window, gc_border, x + x_off, y + y_off,
                     text, strlen (text));
        XDrawString (display, shapemask, gc_shape, x + x_off, y + y_off,
		     text, strlen (text));
      }
    }

    /* draw colored text */
    XDrawString (display, window, gc_text, x + 2, y + 2,
                     text, strlen(text));

    /* add shape mask to window */
    XShapeCombineMask (display, window, ShapeBounding, 0, 0, 
                       shapemask, ShapeSet);

    /* move the window to center the text */
    
    x = 0;
    y = (MSG_H - msg.font_height) / 2 + msg.line_number * MSG_H;

   /* clean up */
   XFreeGC (display, gc_text);
   XFreeGC (display, gc_shape);
   XFreeGC (display, gc_border);
   XFreeFont (display, msg.font_info);
   XFreePixmap (display, shapemask);
    return 0;
  }
}

#ifdef NEW_CODE
int DrawXpm
/* new function; draws given XPM at a given position instead of using
 * msg_t type
 * We should rename this one to DrawXpm and the other one to DrawMsgXpm
 * to be more useful
 */
(Display* display, Window window, char *xpmfile,  int x, int y)
{
  Pixmap pixmap, shapemask;
  XpmAttributes attributes;
  XGCValues gcv;
  GC gc_clip;
  int return_value;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  attributes.valuemask = 0;	/* It is important to set the value mask ! */
  gc_clip = XCreateGC (display, window, GCFunction|GCGraphicsExposures, &gcv);

  DEBUG (("reading pixmap\n"));
  return_value = XpmReadFileToPixmap (display, window, xpmfile, 
  				   &pixmap, &shapemask, &attributes);

  if (return_value != 0)
  {
    printf ("ERROR: XpmReadFileToPixmap failed with value %d\n", return_value);
    return 1;
  }

  DEBUG (("moving pixmap window to %d - %d\n", x, y));
  XMoveWindow (display, window, x, y);
  XShapeCombineMask (display, window, ShapeBounding, 0, 0, shapemask, ShapeSet);
  XCopyArea (display, pixmap, window, gc_clip, 0, 0, 50, 50, 0, 0); //FIXME : get these params right !


  /* clean up stuff */
  XFreeGC (display, gc_clip);
  XFreePixmap (display, pixmap);
  XFreePixmap (display, shapemask);

  return 1;
}
  
int DrawMsgXpm 
(Display* display, Window window, msg_t msg)
/* now call DrawGivenXpmAt
 */
{
 int x, y;

  /* Calculate where to move the window if you want to center it */
 /*
  x = (IMG_W - attributes.width) / 2;
  y = (IMG_H - attributes.height) / 2 + msg.line_number * MSG_H;
  */
  DrawXpm (display, window, msg.icon_file, 0, msg.line_number * MSG_H);

 return 1;
}
#else

int DrawXpm 
(Display* display, Window window, msg_t msg)
{
  Pixmap pixmap, shapemask;
  XpmAttributes attributes;
  XGCValues gcv;
  GC gc_clip;
  int x, y;
  int return_value;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  attributes.valuemask = 0;	/* It is important to set the value mask ! */
  gc_clip = XCreateGC (display, window, GCFunction|GCGraphicsExposures, &gcv);

  DEBUG (("reading pixmap\n"));
  return_value = XpmReadFileToPixmap (display, window, msg.icon_file, 
  				   &pixmap, &shapemask, &attributes);

  if (return_value != 0)
  {
    printf ("ERROR: XpmReadFileToPixmap failed with value %d\n", return_value);
    return 1;
  }

  XShapeCombineMask (display, window, ShapeBounding, 0, 0, shapemask, ShapeSet);
  XCopyArea (display, pixmap, window, gc_clip, 0, 0, 100, 100, 0, 0);

  /* Calculate where to move the window if you want to center it */
  x = (IMG_W - attributes.width) / 2;
  y = (IMG_H - attributes.height) / 2 + msg.line_number * MSG_H;
  DEBUG (("moving pixmap window to %d - %d\n", x, y));
  XMoveWindow (display, window, x, y);

  /* clean up stuff */
  XFreeGC (display, gc_clip);
  XFreePixmap (display, pixmap);
  XFreePixmap (display, shapemask);

  return 1;
}
#endif

int
copy_string (char** dest, char* src, const char* dest_descr)
{
  if (*dest != NULL) free (*dest);
  *dest = (unsigned char *) malloc (strlen (src) + 1);
  if (*dest == NULL)
  { 
    fprintf (stderr, "Could not alloc memory for %s !\n", dest_descr);
    return 1;
  }
  strcpy (*dest, src);
  /* null-terminate */
  (*dest)[strlen(src)] = '\0';
  return 0;
}

void
DefaultMsg (msg_t* msg)
{
/*
 * initialize message with default values
 */

  if (msg->font_name != NULL)
    free (msg->font_name);
  copy_string (&msg->font_name, DEF_FONT, "font name");
  if (msg->color_name != NULL)
    free (msg->color_name);
  copy_string (&msg->color_name, DEF_COLOR, "color name");
  if (msg->text != NULL)
    free (msg->text);
  msg->duration = DEF_DURATION;
  msg->line_number = DEF_LINE_NUMBER;
}

void
ClearMsg (msg_t* msg)
{
  msg->duration = 0;
  msg->line_number = 0;
  msg->font_name = NULL;
  msg->color_name = NULL;
  msg->icon_file = NULL;
  msg->text = NULL;
}

void
FreeMsg (msg_t* msg)
/* free all malloc'ed components of msg */
{
  if (msg == NULL) return;

  if (msg->font_name)  free (msg->font_name);
  if (msg->color_name) free (msg->color_name);
  if (msg->icon_file)  free (msg->icon_file);
  if (msg->text)       free (msg->text);
}

void
KillMsg (Display* display, msg_t* msg)
/* removes a message from the screen and clears all of the data for it */
{
  if (msg->img_w) 
  {
    XUnmapWindow (display, msg->img_w);
    XDestroyWindow (display, msg->img_w);
  }
  if (msg->msg_w)
  {
    XUnmapWindow (display, msg->msg_w);
    XDestroyWindow (display, msg->msg_w);
  }
  XFlush (display);
}
